Module: RobustExcelOle
- Defined in:
- lib/robust_excel_ole/excel.rb,
lib/robust_excel_ole/base.rb,
lib/robust_excel_ole/cell.rb,
lib/robust_excel_ole/range.rb,
lib/robust_excel_ole/cygwin.rb,
lib/robust_excel_ole/version.rb,
lib/robust_excel_ole/list_row.rb,
lib/robust_excel_ole/workbook.rb,
lib/robust_excel_ole/bookstore.rb,
lib/robust_excel_ole/worksheet.rb,
lib/robust_excel_ole/list_object.rb,
lib/robust_excel_ole/vba_objects.rb,
lib/robust_excel_ole/address_tool.rb,
lib/robust_excel_ole/range_owners.rb
Overview
module User32
# Extend this module to an importer
extend Fiddle::Importer
# Load 'user32' dynamic library into this importer
dlload 'user32'
# Set C aliases to this importer for further understanding of function signatures
typealias 'HWND', 'HANDLE'
typealias 'HANDLE', 'void*'
typealias 'LPCSTR', 'const char*'
typealias 'LPCWSTR', 'const wchar_t*'
typealias 'UINT', 'unsigned int'
typealias 'ppvObject', 'void**'
typealias 'DWORD', 'unsigned long'
typealias 'LPDWORD', 'DWORD*'
typealias 'PDWORD_PTR', 'DWORD**'
typealias 'WPARAM', 'UINT*'
typealias 'LPARAM', 'INT*'
typealias 'LRESULT', 'DWORD'
# Import C functions from loaded libraries and set them as module functions
extern 'DWORD GetWindowThreadProcessId(HWND, LPDWORD)'
extern 'HWND FindWindowExA(HWND, HWND, LPCSTR, LPCSTR)'
extern 'DWORD SetForegroundWindow(HWND)'
extern 'LRESULT SendMessageTimeoutA(HWND, UINT, WPARAM, LPARAM, UINT, UINT, PDWORD_PTR)'
end
module Oleacc
# Extend this module to an importer
extend Fiddle::Importer
# Load 'oleacc' dynamic library into this importer
dlload 'oleacc'
# Set C aliases to this importer for further understanding of function signatures
typealias 'HWND', 'HANDLE'
typealias 'HANDLE', 'void*'
typealias 'ppvObject', 'void**'
typealias 'DWORD', 'unsigned long'
typealias 'HRESULT', 'long'
Guid = struct [
'unsigned long data1',
'unsigned short data2',
'unsigned short data3',
'unsigned char data4[8]'
]
# Import C functions from loaded libraries and set them as module functions
extern 'HRESULT AccessibleObjectFromWindow(HWND, DWORD, struct guid*, ppvObject)'
#typealias 'REFIID', 'struct guid*'
#extern 'HRESULT AccessibleObjectFromWindow(HWND, DWORD, REFIID, ppvObject)'
#extern 'HRESULT AccessibleObjectFromWindow(HWND, DWORD, struct GUID*, ppvObject)'
#extern 'HRESULT AccessibleObjectFromWindow(HWND, DWORD, void*, ppvObject)'
#extern 'HRESULT AccessibleObjectFromWindow(HWND, DWORD, struct GUID*, ppvObject)'
end
Defined Under Namespace
Modules: Cygwin Classes: AddressTool, Base, Cell, Excel, ListObject, ListRow, Range, RangeOwners, VbaObjects, Workbook, Worksheet
Constant Summary collapse
- Application =
Excel- VERSION =
"1.39"- TableRow =
ListRow- Book =
Workbook- Sheet =
Worksheet